#!/bin/sh

## CONSTANTS ##

BUILD_NUMBER="09.30.00.01770"
SCRIPT_NAME=` basename $0 | sed 's;\.sh$;;' `
MY_UNAME=` uname | sed 's;SunOS;Solaris;' `
OLD_CD_ROOT="$CD_ROOT"
LOG_FILE=/tmp/${SCRIPT_NAME}.log
RESOURCE_LOC="` dirname $0 `/resources"
MW_PATCH=vmw511_linux_ESRI_RUNTIME_08_Jun_2008.sh

## GLOBAL VARIABLES ##

CD_ROOT=""
PRODUCT_NAME="unknown"
RH5_TEMP_DIR=""
RH5_INSTALL_DIR=""
SETUP_EXIT_CODE=42
SILENT_MODE=false
SILENT_PROP_FILE=""
UNSUPPORTED_PRODUCT="false"

## FUNCTIONS ##

# provides valid exit codes from "which" on all platforms
# expects 1 parameter, the program to search for
# returns 0 if found, 1 if not found, -1 if parameter is missing
which_code()
{
   if [ "x$1" = "x" ]
	then
		echo "which_code requires a program name to search for, i.e. which_code ps" >&2
		return -1
	fi

	WHICH_RESULT=` which "$1" 2>&1 `
	WHICH_EXIT_CODE=$?

	if [ $WHICH_EXIT_CODE -ne 0 ]
	then
		return $WHICH_EXIT_CODE
	else
		GREP_EXIT_CODE=` echo $WHICH_RESULT 2>&1 | grep "no $1" >/dev/null 2>&1; echo $? `
		if [ ` echo $GREP_EXIT_CODE ` -eq 0 ]
		then
			return 1
		else
			return 0
		fi
	fi
}

# print usage instructions for this script
display_usage()
{
	echo "Usage: ` basename $0 ` [-s PROP_FILE] CD_ROOT" >&2
	echo "Installs ArcGIS Server 9.3, ArcIMS 9.3, ArcGIS Engine Runtime 9.3, or" >&2
	echo "ArcReader 9.3 on Red Hat Enterprise Linux 5." >&2
	echo "CD_ROOT is the absolute path to the mounted ArcGIS 9.3 media to install from." >&2
	echo "	-s: install ArcGIS Server \"silently\", using properties file PROP_FILE" >&2
}

# validate and set CD_ROOT if it is valid 9.3 media
validate_set_CD_ROOT()
{
	if [ "x$1X" != "xX" ] && [ -d "$1" ]
	then
		if [ -f $1/ArcGISServer/build.txt ]
		then
			if [ ` cat $1/ArcGISServer/build.txt ` = $BUILD_NUMBER ]
			then
				CD_ROOT=$1
				PRODUCT_NAME=ArcGISServer
			fi
		elif [ -f $1/` echo $MY_UNAME | tr "[:upper:]" "[:lower:]" `/ArcGISServer/build.txt ]
		then
			MY_UNAME_LOWER=` echo $MY_UNAME | tr "[:upper:]" "[:lower:]" `
			if [ ` cat $1/$MY_UNAME_LOWER/ArcGISServer/build.txt ` = $BUILD_NUMBER ]
			then
				CD_ROOT=$1/$MY_UNAME_LOWER
				PRODUCT_NAME=ArcGISServer
			fi
		elif [ -f $1/ArcIMS/Merge/build.txt ]
		then
			if [ ` cat $1/ArcIMS/Merge/build.txt ` = $BUILD_NUMBER ]
			then
				CD_ROOT=$1
				PRODUCT_NAME=ArcIMS
			fi
		elif [ -f $1/ArcIMS_${MY_UNAME}/ArcIMS/Merge/build.txt ]
		then
			if [ ` cat $1/ArcIMS_${MY_UNAME}/ArcIMS/Merge/build.txt ` = $BUILD_NUMBER ]
			then
				CD_ROOT=$1/ArcIMS_${MY_UNAME}
				PRODUCT_NAME=ArcIMS
			fi
		else
			for i in ArcReader ArcGISEngineRT
			do
				if [ -f $1/$i/Merge/build.txt ]
				then
					if [ ` cat $1/$i/Merge/build.txt ` = $BUILD_NUMBER ]
					then
						CD_ROOT=$1
						PRODUCT_NAME="$i"
					fi
				elif [ -f $1/${MY_UNAME}/$i/Merge/build.txt ]
				then
					if [ ` cat $1/${MY_UNAME}/$i/Merge/build.txt ` = $BUILD_NUMBER ]
					then
						CD_ROOT=$1/${MY_UNAME}
						PRODUCT_NAME="$i"
					fi
				fi
			done
		fi

		# check if it is not a supported product
		if [ "x${CD_ROOT}X" = "xX" ]
		then
			if [ -x $1/Setup ]
			then
				CD_ROOT=$1
				UNSUPPORTED_PRODUCT=true
			elif [ -x $1/${MY_UNAME}/Setup ]
			then
				CD_ROOT=$1/${MY_UNAME}
				UNSUPPORTED_PRODUCT=true
			elif [ -x $1/SetupArcIMS ]
			then
				CD_ROOT=$1
				UNSUPPORTED_PRODUCT=true
				PRODUCT_NAME="ArcIMS"
			elif [ -x $1/${MY_UNAME}/SetupArcIMS ]
			then
              CD_ROOT=$1/${MY_UNAME}
				UNSUPPORTED_PRODUCT=true
				PRODUCT_NAME="ArcIMS"
			fi
		fi
	fi

	echo "" >> $LOG_FILE
	echo "PRODUCT_NAME=$PRODUCT_NAME" >> $LOG_FILE

	if [ "x${CD_ROOT}X" != "xX" ]
	then
		return 0
	else
		return 1
	fi
}

# determine RH5_INSTALL_DIR
set_RH5_install_dir()
{
	if [ "x${RH5_INSTALL_DIR}X" = "xX" ]
	then
		if [ $PRODUCT_NAME = "ArcGISServer" ]
		then
			RH5_INSTALL_DIR=` mwadm status 2>/dev/null | grep "Core Data Directory" | sed -e 's;^.* : ;;' -e 's;/servercore/mwcore$;;' `
		else
			RH5_INSTALL_DIR=` grep USER_INSTALL_DIR $HOME/${PRODUCT_NAME}_InstallLog.log 2>/dev/null | sed 's;^USER_INSTALL_DIR=;;' `
			
			if [ "x${RH5_INSTALL_DIR}X" = "xX" ]
			then
				ESRI_PROP_FILE=ESRI.properties.` hostname `
				RH5_INSTALL_DIR=` grep Z_${PRODUCT_NAME}_INSTALL_DIR $HOME/$ESRI_PROP_FILE 2>/dev/null | sed 's;^.*_INSTALL_DIR=;;' `
			fi
		fi
	fi
}


install_MW_patch()
{
		echo >> $LOG_FILE
		echo "Copying MW patch files..." >> $LOG_FILE
		set_RH5_install_dir
		
		if [ "x${RH5_INSTALL_DIR}X" != "xX" ]
		then
			if [ $PRODUCT_NAME = "ArcGISServer" ]
			then
				# determine Server user
				SERVER_USER=` mwadm status | grep "MSC Administrator" | sed -e 's;^.*: ;;' `
				# make LOG_FILE writable by SERVER_USER
				chown $SERVER_USER $LOG_FILE
				echo "SERVER_USER=$SERVER_USER" >> $LOG_FILE
			
				# copy extracted MW patch files
				MW_HOME=${RH5_INSTALL_DIR}/servercore/vmw
				echo "MW_HOME=$MW_HOME" >> $LOG_FILE
				cp -Rv $RESOURCE_LOC/vmw/* $MW_HOME >> $LOG_FILE 2>&1
				chown -R $SERVER_USER $MW_HOME
			else
				# copy extracted MW patch files
				MW_HOME=$RH5_INSTALL_DIR/vmw
				echo "MW_HOME=$MW_HOME" >> $LOG_FILE
				cp -Rv $RESOURCE_LOC/vmw/* $MW_HOME >> $LOG_FILE 2>&1
			fi
		else
			echo "ERROR: Cannot install MW patch. RH5_INSTALL_DIR not set." >> $LOG_FILE
		fi
}

# waits until init_Xvfb.sh exists, then adds a fontpath to the Linux Xvfb command
# waits for MainWin to be fully installed and applies the MW patch
init_xvfb_MW_loop()
{
	# wait for MW to start so we can get the install directory
	while [ ` mwadm status 2>/dev/null | grep "Core Data Directory" >/dev/null 2>&1; echo $? ` -ne 0 ]  
	do
		sleep 10
	done
	set_RH5_install_dir
	
	# wait for init_Xvfb.sh to get installed before modifying it
	while [ ! -f $RH5_INSTALL_DIR/servercore/.Server/init_Xvfb.sh ]
	do
		sleep 10
	done
	
	# add fontpath to Linux Xvfb invocation line
	mv $RH5_INSTALL_DIR/servercore/.Server/init_Xvfb.sh $RH5_INSTALL_DIR/servercore/.Server/init_Xvfb.sh.orig
	cat $RH5_INSTALL_DIR/servercore/.Server/init_Xvfb.sh.orig | sed -e 's;-ac -sp;-fp /usr/share/X11/fonts/misc -ac -sp;' >> $RH5_INSTALL_DIR/servercore/.Server/init_Xvfb.sh
	rm -f $RH5_INSTALL_DIR/servercore/.Server/init_Xvfb.sh.orig
	if [ ` grep "\-fp /usr/share/X11/fonts/misc" $RH5_INSTALL_DIR/servercore/.Server/init_Xvfb.sh >/dev/null 2>&1; echo $? ` -eq 0 ]
	then
		echo "Added fontpath to init_Xvfb.sh" >> $LOG_FILE
	else
		echo "Adding fontpath to init_Xvfb.sh FAILED" >> $LOG_FILE
	fi
	
	# wait for mw location to be created
	while [ ! -f $RH5_INSTALL_DIR/servercore/vmw/mw/bin/mwversion ]
	do
		sleep 10
	done

	# MW patch keeps saying it cannot find mwversion
	sleep 10
	# ls -l $RH5_INSTALL_DIR/servercore/vmw/mw/bin/mwversion >> $LOG_FILE
	install_MW_patch
}

# modified from InstallServerSilently.sh script on 9.3 media
install_Server_silently()
{
	if [ $# -eq 0 ]
	then
   	echo "   "
		echo "Usage:    $0 /path/to/properties/file" 1>&2
		echo "   "
		exit 1
	fi

	propfile=$1
	if [ ! -f $propfile ]
	then
		echo "   "
		echo "ERROR: The properties file provided does not exist:" | tee -a $LOG_FILE 1>&2
		echo "   $propfile"  | tee -a $LOG_FILE 1>&2
		echo "   "
		exit 1
	fi

	if [ `dirname $propfile` = "." ]
	then
		propfile=`pwd`/`basename $propfile`
	fi

	# Launch the setup
	echo ""
	echo "Executing $CD_ROOT/ArcGISServer/$MY_UNAME/Disk1/InstData/VM/install.bin -i silent -f $propfile" >> $LOG_FILE
	{
		export CD_ROOT
		sh $CD_ROOT/ArcGISServer/$MY_UNAME/Disk1/InstData/VM/install.bin -i silent -f $propfile 2>&1
		echo $? > $RH5_TEMP_DIR/setup_exit_code.txt
	} | tee -a $LOG_FILE
}

## MAIN ##

if [ $MY_UNAME != "Linux" ] && [ $MY_UNAME != "Solaris" ]
then
	echo "Unrecognized platform: $MY_UNAME" >&2
	display_usage
	exit 1
fi	

date > $LOG_FILE
echo "Command: $0 $*" >> $LOG_FILE
echo >> $LOG_FILE

# check arguments
while getopts :hs: opt
do
	case $opt in
	h) display_usage
		exit 0
		;;
	s) SILENT_MODE=true
		if [ -f $OPTARG ]
		then
			SILENT_PROP_FILE=$OPTARG
		else
			echo "Cannot find properties file $OPTARG" | tee -a $LOG_FILE >&2
			exit 1
		fi	
		;;
	'?') echo "$0: Invalid option -$OPTARG" >&2
		display_usage
		exit 1
		;;
	esac
done

if [ ` expr $OPTIND - 1 ` != 0 ]
then
	shift ` expr $OPTIND - 1 `
fi
	

# using passed argument for CD_ROOT
if [ "x$1X" != "xX" ]
then
	validate_set_CD_ROOT $1
	if [ $? -eq 0 ]
	then
		echo "CD_ROOT=$CD_ROOT (from passed argument)" >> $LOG_FILE
	fi
# using CD_ROOT from environment
elif [ "x${OLD_CD_ROOT}X" != "xX" ]
then
	validate_set_CD_ROOT $OLD_CD_ROOT
	if [ $? -eq 0 ]
	then
		echo "CD_ROOT=$CD_ROOT (from environment variable)" >> $LOG_FILE
	fi
else
	# check if the script is running inside CD_ROOT 
	validate_set_CD_ROOT $PWD
	if [ $? -eq 0 ]
	then
		echo "CD_ROOT=$CD_ROOT (from current working directory)" >> $LOG_FILE
	else
		# check if we are installing from RH5 support media
		validate_set_CD_ROOT "` dirname $0 `"
		if [ $? -eq 0 ]
		then
			echo "CD_ROOT=$CD_ROOT (from script location)" >> $LOG_FILE
		fi
	fi

	if [ "x${CD_ROOT}X" = "xX" ]
	then
		# check default CD mount points
		for cdrom in /media/cdrom /media/cdrecorder /media/ESRI /mnt/cdrom
		do
			validate_set_CD_ROOT "$cdrom"
			if [ $? -eq 0 ]
			then
				echo "CD_ROOT=$CD_ROOT (from default CD mount points)" >> $LOG_FILE
			fi
		done
	fi
fi	

if [ "x${CD_ROOT}X" = "xX" ]
then
	echo "Could not find valid ArcGIS 9.3 media!" >&2
	echo "" >&2
	display_usage
	exit 1
fi

# verify RH5
echo >> $LOG_FILE
RH5_OS=false
if [ -f /etc/redhat-release ]
then
	REDHAT_RELEASE_TEXT="` cat /etc/redhat-release `"
	echo "REDHAT_RELEASE_TEXT=$REDHAT_RELEASE_TEXT" >> $LOG_FILE
	if [ ` echo "$REDHAT_RELEASE_TEXT" | grep "^Red Hat Enterprise Linux Server release 5" >/dev/null 2>&1; echo $? ` -eq 0 ]
	then
		RH5_OS=true
	fi
else
	echo "WARNING: /etc/redhat-release does not exist!" >> $LOG_FILE
	if [ $MY_UNAME = "Linux" ] && [ -f /etc/issue ]
	then
		echo "/etc/issue: ` cat /etc/issue `" >> $LOG_FILE
	else
		echo "uname: ` uname `" >> $LOG_FILE
	fi
fi

unset AWT_TOOLKIT

# if not RH5, just do normal CD_ROOT/Setup
echo >> $LOG_FILE
if [ $RH5_OS != "true" ] || [ $UNSUPPORTED_PRODUCT = "true" ]
then
	if [ $PRODUCT_NAME = "ArcGISServer" ] && [ $SILENT_MODE = "true" ]
	then
		install_Server_silently $SILENT_PROP_FILE
	elif [ $PRODUCT_NAME = "ArcIMS" ]
	then
		echo "Executing $CD_ROOT/SetupArcIMS" >> $LOG_FILE
		{
			$CD_ROOT/SetupArcIMS
			echo $? > /tmp/setup_exit_code.txt
		} | tee -a $LOG_FILE
	else
		echo "Executing $CD_ROOT/Setup" >> $LOG_FILE
		{
			$CD_ROOT/Setup
			echo $? > /tmp/setup_exit_code.txt
		} | tee -a $LOG_FILE
	fi
	if [ -f /tmp/setup_exit_code.txt ]
	then
		SETUP_EXIT_CODE=` cat /tmp/setup_exit_code.txt | tr -d '\n' `
		rm -f /tmp/setup_exit_code.txt 2>/dev/null
	fi
else	# RH5 specific
	# check for new/changed libs
	echo "Checking System Requirements..." >> $LOG_FILE
	for package in compat-libf2c libXp compat-libstdc++-33
	do
		RPM_RESULT=` rpm -qa --qf '%{N}-%{V}-%{R}.%{arch}\n' | grep "^${package}[^a-zA-Z]" | grep -v x86_64 `
		echo $RPM_RESULT >> $LOG_FILE
		if [ "x${RPM_RESULT}X" = "xX" ]
		then
			echo "ERROR: ${package} rpm (32-bit) must be installed before continuing Setup." | tee -a $LOG_FILE >&2
			exit 1
		fi
	done

	# create location for temporary files 
	if [ ` which_code mktemp; echo $? ` -eq 0 ]
	then
   	RH5_TEMP_DIR=`mktemp -td ${SCRIPT_NAME}XXXXXXXX`
	else
   	if [ "x$TMPDIR" != "x" ] && [ -d $TMPDIR ] && [ -w $TMPDIR ]
   	then
   	   RH5_TEMP_DIR=/$TMPDIR/${SCRIPT_NAME}$$` perl -e 'print time()' `
   	else
   	   RH5_TEMP_DIR=/tmp/${SCRIPT_NAME}$$` perl -e 'print time()' `
   	fi

   	mkdir $RH5_TEMP_DIR
   	chmod 700 $RH5_TEMP_DIR
	fi

	# remove temp dir on script exit
	trap 'rm -rf $RH5_TEMP_DIR' EXIT
	
	# replace platform_requirements.csv
	mkdir -p $RH5_TEMP_DIR/CD_ROOT/$PRODUCT_NAME/mwTools/sms
	cp $RESOURCE_LOC/platform_requirements.csv $RH5_TEMP_DIR/CD_ROOT/$PRODUCT_NAME/mwTools/sms/
	# create linked tree from real CD_ROOT
	if [ ` which_code lndir; echo $? ` -eq 0 ]
	then
		lndir $CD_ROOT/ $RH5_TEMP_DIR/CD_ROOT/ >$RH5_TEMP_DIR/lndir.log 2>&1
	else
		# create fake lndir
		echo "" >> $LOG_FILE
		echo "Cannot find lndir. Creating linked CD_ROOT manually. " >> $LOG_FILE
		SED_COMMAND="sed -e s;^$CD_ROOT/;;"
		# first create directory structure
		for find_i in ` find $CD_ROOT -type d | grep -v "${CD_ROOT}$" | $SED_COMMAND `
		do
			if [ ! -d $RH5_TEMP_DIR/CD_ROOT/$find_i ]
			then
				mkdir -pv $RH5_TEMP_DIR/CD_ROOT/$find_i >> $LOG_FILE 2>&1
			fi
		done
		# next fill dir tree with links to files
		for find_i in ` find $CD_ROOT -type f | $SED_COMMAND `
		do
			if [ ! -f $RH5_TEMP_DIR/CD_ROOT/$find_i ]
			then
				ln -sv $CD_ROOT/$find_i $RH5_TEMP_DIR/CD_ROOT/$find_i >> $LOG_FILE 2>&1
			fi
		done
	fi
	CD_ROOT=$RH5_TEMP_DIR/CD_ROOT

	echo >> $LOG_FILE
	# IMS only
	if [ $PRODUCT_NAME = "ArcIMS" ]
	then
		# create Xvfb link
		XVFB_LOC=` which Xvfb 2>/dev/null `
		if [ -L /usr/X11R6/bin/Xvfb ] || [ -L /usr/bin/X11/Xvfb ]
		then
			XVFB_LINKED=true
		else
			XVFB_LINKED=false
		fi
		if [ "x${XVFB_LOC}X" != "xX" ] && [ $XVFB_LOC != "/usr/bin/X11/Xvfb" ] && [ $XVFB_LOC != "/usr/X11R6/bin/Xvfb" ] && [ -e $XVFB_LOC ]
		then
			echo "Creating Xvfb link..." >> $LOG_FILE
			for ln_loc in /usr/bin/X11 /usr/X11R6/bin
			do
				if [ $XVFB_LINKED != "true" ] && [ -d $ln_loc ]
				then
					if [ -w $ln_loc ]
					then
						if ln -s $XVFB_LOC ${ln_loc}/Xvfb
						then
							XVFB_LINKED=true
							echo "${ln_loc}/Xvfb link created" >> $LOG_FILE
						fi
					fi
				fi
			done
		
			if [ $XVFB_LINKED != "true" ]
			then
				EXIT_INPUT_LOOP=false
				until [ $EXIT_INPUT_LOOP = "true" ]
				do
					echo "ArcMap Server expects Xvfb to be installed in /usr/bin/X11/ or /usr/X11R6/bin/."
					echo "On this machine, Xvfb is at $XVFB_LOC."
					echo "Would you like this Setup to create a link automatically? [y]/n"
					echo "Note: You will need to enter the root password"
					read LINK_INPUT

					if [ "x${LINK_INPUT}X" = "xX" ] || [ ` echo $LINK_INPUT | grep -i "^y" >/dev/null 2>&1; echo $? ` -eq 0 ]
					then
						SU_EXIT_CODE=""
						echo "Please enter root password."
						if [ ! -d /usr/X11R6/bin ]
						then
							su - -c "mkdir -p /usr/X11R6/bin; ln -s $XVFB_LOC /usr/X11R6/bin/Xvfb"
							echo "Executing su - -c \"mkdir -p /usr/X11R6/bin; ln -s $XVFB_LOC /usr/X11R6/bin/Xvfb\"" >> $LOG_FILE
							SU_EXIT_CODE=$?
						else
							su - -c "ln -s $XVFB_LOC /usr/X11R6/bin/Xvfb"
							echo "Executing su - -c \"ln -s $XVFB_LOC /usr/X11R6/bin/Xvfb\"" >> $LOG_FILE
							SU_EXIT_CODE=$?
						fi
						
						if [ $SU_EXIT_CODE -eq 0 ] && [ -L /usr/X11R6/bin/Xvfb ]
						then
							echo "/usr/X11R6/bin/Xvfb link created as root" >> $LOG_FILE
							EXIT_INPUT_LOOP=true
						fi
					elif [ ` echo $LINK_INPUT | grep -i "^n" >/dev/null 2>&1; echo $? ` -eq 0 ]
					then
						echo "User chose not to create Xvfb link automatically." >> $LOG_FILE
						EXIT_INPUT_LOOP=true
					fi
				done
			fi
		else
			echo "Xvfb not in path. Unable to create link." >> $LOG_FILE
		fi
		
		echo ""
	   echo "Executing $CD_ROOT/SetupArcIMS" >> $LOG_FILE
	   {
			$CD_ROOT/SetupArcIMS
			echo $? > $RH5_TEMP_DIR/setup_exit_code.txt
		} | tee -a $LOG_FILE
	# Server only
	elif [ $PRODUCT_NAME = "ArcGISServer" ]
	then
		echo "Starting Server steps" >> $LOG_FILE
		# check for /usr/X11R6/lib/X11/rgb
		# create dir if necessary and chmod at least 744 permissions
		if [ ! -d /usr/X11R6/lib/X11/rgb ]
		then
			if mkdir -p /usr/X11R6/lib/X11/rgb
			then
				echo "/usr/X11R6/lib/X11/rgb/ created" >> $LOG_FILE
			else
				echo "/usr/X11R6/lib/X11/rgb/ creation FAILED" >> $LOG_FILE
			fi
		else	
				echo "/usr/X11R6/lib/X11/rgb/ exists" >> $LOG_FILE
		fi
		if chmod a+r /usr/X11R6/lib/X11/rgb
		then
			echo "/usr/X11R6/lib/X11/rgb/ read permissions set" >> $LOG_FILE
		else
			echo "/usr/X11R6/lib/X11/rgb/ read permissions set FAILED" >> $LOG_FILE
		fi
		if chmod a+w /usr/X11R6/lib/X11/rgb
		then
			echo "/usr/X11R6/lib/X11/rgb/ write permissions set" >> $LOG_FILE
		else
			echo "/usr/X11R6/lib/X11/rgb/ write permissions set FAILED" >> $LOG_FILE
		fi

		# backgrounded loop until init_Xvfb.sh installed
		# adds fontpath, installs MW patch
		init_xvfb_MW_loop &
		INIT_XVFB_MW_PID=$!
		trap 'kill -9 $INIT_XVFB_MW_PID 2>/dev/null' EXIT

		# run Setup
		if [ $SILENT_MODE = "true" ]
		then
			install_Server_silently $SILENT_PROP_FILE
		else
			echo ""
			echo "Executing $CD_ROOT/Setup" >> $LOG_FILE
			$CD_ROOT/Setup | tee -a $LOG_FILE
		fi	

		# check server install log and set SETUP_EXIT_CODE accordingly
		set_RH5_install_dir
		if [ ` grep "Installation: Successful." $HOME/ArcGISServer_InstallLog.log >/dev/null 2>&1; echo $? ` -eq 0 ] && [ ` grep "Post installation completed successfully." $RH5_INSTALL_DIR/logs/Setup/configuration_logs/pibinary.log >/dev/null 2>&1; echo $? ` -eq 0 ]
		then
			SETUP_EXIT_CODE=0
		else
			SETUP_EXIT_CODE=1
		fi
	else
		# ArcReader and ArcGISEngineRT
		echo "Executing $CD_ROOT/Setup" >> $LOG_FILE
		{
			$CD_ROOT/Setup
			echo $? > ${RH5_TEMP_DIR}/setup_exit_code.txt
		} | tee -a $LOG_FILE
	fi
	
	# record Setup exit status in LOG_FILE
	if [ -f ${RH5_TEMP_DIR}/setup_exit_code.txt ]
	then
		SETUP_EXIT_CODE=` cat ${RH5_TEMP_DIR}/setup_exit_code.txt | tr -d '\n' `
	fi

	echo >> $LOG_FILE
	if [ $SETUP_EXIT_CODE -eq 0 ]
	then
		echo "Setup completed successfully" >> $LOG_FILE
	else
		echo "Setup exited with exit code $SETUP_EXIT_CODE" >> $LOG_FILE
	fi

	# if not Server, apply MW patch (Server already has it by now)
	if [ $PRODUCT_NAME != "ArcGISServer" ] && [ $SETUP_EXIT_CODE -eq 0 ]
	then
		install_MW_patch
	fi

	# move log from /tmp to RH5_INSTALL_DIR/.Setup/
	set_RH5_install_dir
	if [ "x${RH5_INSTALL_DIR}X" != "xX" ]
	then
		LOG_NAME=${PRODUCT_NAME}_` basename $LOG_FILE `
		mv $LOG_FILE ${RH5_INSTALL_DIR}/.Setup/$LOG_NAME 2>/dev/null
	
		# update uninstall script to remove log
		if [ $PRODUCT_NAME = "ArcGISServer" ]
		then
			UNINSTALL_SCRIPT=${RH5_INSTALL_DIR}/scripts/uninstallArcGISServer
		elif [ $PRODUCT_NAME = "ArcIMS" ]
		then
			UNINSTALL_SCRIPT=${RH5_INSTALL_DIR}/arcims/tools/uninstallArcIMS
		else
			UNINSTALL_SCRIPT=${RH5_INSTALL_DIR}/uninstall${PRODUCT_NAME}
		fi

		if [ -f ${UNINSTALL_SCRIPT} ]
		then
			echo "rm -f ${RH5_INSTALL_DIR}/.Setup/$LOG_NAME >/dev/null 2>&1; rmdir ${RH5_INSTALL_DIR}/.Setup >/dev/null 2>&1; rmdir $RH5_INSTALL_DIR >/dev/null 2>&1" >> ${UNINSTALL_SCRIPT}
		fi
	fi
fi

exit $SETUP_EXIT_CODE
